"""
Tests for CLI.
"""
import os
import shlex
import subprocess
from pathlib import Path

import argbind
import numpy as np
from audiotools import AudioSignal

from dac.__main__ import run


def make_fake_data(data_dir=Path(__file__).parent / "assets"):
    data_dir.mkdir(exist_ok=True, parents=True)
    input_dir = data_dir / "input"
    input_dir.mkdir(exist_ok=True, parents=True)

    for i in range(100):
        signal = AudioSignal(np.random.randn(44_100 * 5), 44_100)
        signal.write(input_dir / f"sample_{i}.wav")
    return input_dir


def make_fake_data_tree():
    data_dir = Path(__file__).parent / "assets"

    for relative_dir in [
        "train/speech",
        "train/music",
        "train/env",
        "val/speech",
        "val/music",
        "val/env",
        "test/speech",
        "test/music",
        "test/env",
    ]:
        leaf_dir = data_dir / relative_dir
        leaf_dir.mkdir(exist_ok=True, parents=True)
        make_fake_data(leaf_dir)
    return {
        split: {
            key: [str(data_dir / f"{split}/{key}")]
            for key in ["speech", "music", "env"]
        }
        for split in ["train", "val", "test"]
    }


def setup_module(module):
    # Make fake dataset dir
    input_datasets = make_fake_data_tree()
    repo_root = Path(__file__).parent.parent

    # Load baseline conf and modify it for testing
    conf = argbind.load_args(repo_root / "conf" / "ablations" / "baseline.yml")

    for key in ["train", "val", "test"]:
        conf[f"{key}/build_dataset.folders"] = input_datasets[key]
    conf["num_iters"] = 1
    conf["val/AudioDataset.n_examples"] = 1
    conf["val_idx"] = [0]
    conf["val_batch_size"] = 1

    argbind.dump_args(conf, Path(__file__).parent / "assets" / "conf.yml")


def teardown_module(module):
    repo_root = Path(__file__).parent.parent
    # Remove fake dataset dir
    subprocess.check_output(["rm", "-rf", f"{repo_root}/tests/assets"])
    subprocess.check_output(["rm", "-rf", f"{repo_root}/tests/runs"])


def test_single_gpu_train():
    env = os.environ.copy()
    env["CUDA_VISIBLE_DEVICES"] = "0"
    repo_root = Path(__file__).parent.parent
    args = shlex.split(
        f"python {repo_root}/scripts/train.py --args.load {repo_root}/tests/assets/conf.yml --save_path {repo_root}/tests/runs/baseline"
    )
    subprocess.check_output(args, env=env)


def test_multi_gpu_train():
    env = os.environ.copy()
    env["CUDA_VISIBLE_DEVICES"] = "0,1"
    repo_root = Path(__file__).parent.parent
    args = shlex.split(
        f"torchrun --nproc_per_node gpu {repo_root}/scripts/train.py --args.load {repo_root}/tests/assets/conf.yml --save_path {repo_root}/tests/runs/baseline_multigpu"
    )
    subprocess.check_output(args, env=env)



"""
Tests for CLI.
"""
import subprocess
from pathlib import Path

import argbind
import numpy as np
import pytest
import torch
from audiotools import AudioSignal

from dac.__main__ import run


def setup_module(module):
    data_dir = Path(__file__).parent / "assets"
    data_dir.mkdir(exist_ok=True, parents=True)
    input_dir = data_dir / "input"
    input_dir.mkdir(exist_ok=True, parents=True)

    for i in range(5):
        signal = AudioSignal(np.random.randn(1000), 44_100)
        signal.write(input_dir / f"sample_{i}.wav")
    return input_dir


def teardown_module(module):
    repo_root = Path(__file__).parent.parent
    subprocess.check_output(["rm", "-rf", f"{repo_root}/tests/assets"])


@pytest.mark.parametrize("model_type", ["44khz", "24khz", "16khz"])
def test_reconstruction(model_type):
    # Test encoding
    input_dir = Path(__file__).parent / "assets" / "input"
    output_dir = input_dir.parent / model_type / "encoded_output"
    args = {
        "input": str(input_dir),
        "output": str(output_dir),
        "device": "cuda" if torch.cuda.is_available() else "cpu",
        "model_type": model_type,
    }
    with argbind.scope(args):
        run("encode")

    # Test decoding
    input_dir = output_dir
    output_dir = input_dir.parent / model_type / "decoded_output"
    args = {
        "input": str(input_dir),
        "output": str(output_dir),
        "model_type": model_type,
    }
    with argbind.scope(args):
        run("decode")


def test_compression():
    # Test encoding
    input_dir = Path(__file__).parent / "assets" / "input"
    output_dir = input_dir.parent / "encoded_output_quantizers"
    args = {
        "input": str(input_dir),
        "output": str(output_dir),
        "n_quantizers": 3,
        "device": "cuda" if torch.cuda.is_available() else "cpu",
    }
    with argbind.scope(args):
        run("encode")

    # Open .dac file
    dac_file = output_dir / "sample_0.dac"
    artifacts = np.load(dac_file, allow_pickle=True)[()]
    codes = artifacts["codes"]

    # Ensure that the number of quantizers is correct
    assert codes.shape[1] == 3

    # Ensure that dtype of compression is uint16
    assert codes.dtype == np.uint16


# CUDA_VISIBLE_DEVICES=0 python -m pytest tests/test_cli.py -s



from setuptools import find_packages
from setuptools import setup

with open("README.md") as f:
    long_description = f.read()

setup(
    name="descript-audio-codec-unofficial",
    version="1.0.0",
    classifiers=[
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Programming Language :: Python :: 3.7",
        "Topic :: Artistic Software",
        "Topic :: Multimedia",
        "Topic :: Multimedia :: Sound/Audio",
        "Topic :: Multimedia :: Sound/Audio :: Editors",
        "Topic :: Software Development :: Libraries",
    ],
    description="A high-quality general neural audio codec.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Prem Seetharaman, Rithesh Kumar",
    author_email="prem@descript.com",
    url="https://github.com/descriptinc/descript-audio-codec",
    license="MIT",
    packages=find_packages(),
    keywords=["audio", "compression", "machine learning"],
    install_requires=[
        "argbind>=0.3.7",
        "descript-audiotools-unofficial",
        "einops",
        "numpy",
        "torch",
        "torchaudio",
        "tqdm",
    ],
    extras_require={
        "dev": [
            "pytest",
            "pytest-cov",
            "pynvml",
            "psutil",
            "pandas",
            "onnx",
            "onnx-simplifier",
            "seaborn",
            "jupyterlab",
            "pandas",
            "watchdog",
            "pesq",
            "tabulate",
            "encodec",
        ],
    },
)
